(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = global || self, factory(global.egraph = {}));
}(this, function (exports) { 'use strict';

    let wasm;

    const heap = new Array(32);

    heap.fill(undefined);

    heap.push(undefined, null, true, false);

    let heap_next = heap.length;

    function addHeapObject(obj) {
        if (heap_next === heap.length) heap.push(heap.length + 1);
        const idx = heap_next;
        heap_next = heap[idx];

        heap[idx] = obj;
        return idx;
    }
    function __wbg_elem_binding0(arg0, arg1, arg2, arg3, arg4) {
        wasm.__wbg_function_table.get(158)(arg0, arg1, addHeapObject(arg2), arg3, addHeapObject(arg4));
    }

    function getObject(idx) { return heap[idx]; }

    function dropObject(idx) {
        if (idx < 36) return;
        heap[idx] = heap_next;
        heap_next = idx;
    }

    function takeObject(idx) {
        const ret = getObject(idx);
        dropObject(idx);
        return ret;
    }
    /**
    * @param {any} graph
    * @returns {any}
    */
    function longestPathRanking(graph) {
        const ret = wasm.longestPathRanking(addHeapObject(graph));
        return takeObject(ret);
    }

    /**
    * @param {any} graph
    * @param {number} size
    * @returns {any}
    */
    function minWidthRanking(graph, size) {
        const ret = wasm.minWidthRanking(addHeapObject(graph), size);
        return takeObject(ret);
    }

    let stack_pointer = 32;

    function addBorrowedObject(obj) {
        if (stack_pointer == 1) throw new Error('out of js stack');
        heap[--stack_pointer] = obj;
        return stack_pointer;
    }

    function _assertClass(instance, klass) {
        if (!(instance instanceof klass)) {
            throw new Error(`expected instance of ${klass.name}`);
        }
        return instance.ptr;
    }
    /**
    * @param {any} graph
    * @param {any} grouping
    * @param {any} weight
    * @returns {any}
    */
    function aggregateNodes(graph, grouping, weight) {
        const ret = wasm.aggregateNodes(addHeapObject(graph), addHeapObject(grouping), addHeapObject(weight));
        return takeObject(ret);
    }

    /**
    * @param {any} graph
    * @param {any} grouping
    * @param {any} weight
    * @returns {any}
    */
    function aggregateEdges(graph, grouping, weight) {
        const ret = wasm.aggregateEdges(addHeapObject(graph), addHeapObject(grouping), addHeapObject(weight));
        return takeObject(ret);
    }

    /**
    * @param {any} graph
    * @returns {any}
    */
    function connected_components(graph) {
        const ret = wasm.connected_components(addHeapObject(graph));
        return takeObject(ret);
    }

    /**
    * @param {any} graph
    * @param {any} source
    * @param {any} target
    * @param {number} mu
    * @returns {any}
    */
    function muQuasiBicliques(graph, source, target, mu) {
        const ret = wasm.muQuasiBicliques(addHeapObject(graph), addHeapObject(source), addHeapObject(target), mu);
        return takeObject(ret);
    }

    let cachedTextDecoder = new TextDecoder('utf-8');

    let cachegetUint8Memory = null;
    function getUint8Memory() {
        if (cachegetUint8Memory === null || cachegetUint8Memory.buffer !== wasm.memory.buffer) {
            cachegetUint8Memory = new Uint8Array(wasm.memory.buffer);
        }
        return cachegetUint8Memory;
    }

    function getStringFromWasm(ptr, len) {
        return cachedTextDecoder.decode(getUint8Memory().subarray(ptr, ptr + len));
    }

    function handleError(e) {
        wasm.__wbindgen_exn_store(addHeapObject(e));
    }

    let WASM_VECTOR_LEN = 0;

    let cachedTextEncoder = new TextEncoder('utf-8');

    let passStringToWasm;
    if (typeof cachedTextEncoder.encodeInto === 'function') {
        passStringToWasm = function(arg) {


            let size = arg.length;
            let ptr = wasm.__wbindgen_malloc(size);
            let offset = 0;
            {
                const mem = getUint8Memory();
                for (; offset < arg.length; offset++) {
                    const code = arg.charCodeAt(offset);
                    if (code > 0x7F) break;
                    mem[ptr + offset] = code;
                }
            }

            if (offset !== arg.length) {
                arg = arg.slice(offset);
                ptr = wasm.__wbindgen_realloc(ptr, size, size = offset + arg.length * 3);
                const view = getUint8Memory().subarray(ptr + offset, ptr + size);
                const ret = cachedTextEncoder.encodeInto(arg, view);

                offset += ret.written;
            }
            WASM_VECTOR_LEN = offset;
            return ptr;
        };
    } else {
        passStringToWasm = function(arg) {


            let size = arg.length;
            let ptr = wasm.__wbindgen_malloc(size);
            let offset = 0;
            {
                const mem = getUint8Memory();
                for (; offset < arg.length; offset++) {
                    const code = arg.charCodeAt(offset);
                    if (code > 0x7F) break;
                    mem[ptr + offset] = code;
                }
            }

            if (offset !== arg.length) {
                const buf = cachedTextEncoder.encode(arg.slice(offset));
                ptr = wasm.__wbindgen_realloc(ptr, size, size = offset + buf.length);
                getUint8Memory().set(buf, ptr + offset);
                offset += buf.length;
            }
            WASM_VECTOR_LEN = offset;
            return ptr;
        };
    }

    let cachegetInt32Memory = null;
    function getInt32Memory() {
        if (cachegetInt32Memory === null || cachegetInt32Memory.buffer !== wasm.memory.buffer) {
            cachegetInt32Memory = new Int32Array(wasm.memory.buffer);
        }
        return cachegetInt32Memory;
    }

    function debugString(val) {
        // primitive types
        const type = typeof val;
        if (type == 'number' || type == 'boolean' || val == null) {
            return  `${val}`;
        }
        if (type == 'string') {
            return `"${val}"`;
        }
        if (type == 'symbol') {
            const description = val.description;
            if (description == null) {
                return 'Symbol';
            } else {
                return `Symbol(${description})`;
            }
        }
        if (type == 'function') {
            const name = val.name;
            if (typeof name == 'string' && name.length > 0) {
                return `Function(${name})`;
            } else {
                return 'Function';
            }
        }
        // objects
        if (Array.isArray(val)) {
            const length = val.length;
            let debug = '[';
            if (length > 0) {
                debug += debugString(val[0]);
            }
            for(let i = 1; i < length; i++) {
                debug += ', ' + debugString(val[i]);
            }
            debug += ']';
            return debug;
        }
        // Test for built-in
        const builtInMatches = /\[object ([^\]]+)\]/.exec(toString.call(val));
        let className;
        if (builtInMatches.length > 1) {
            className = builtInMatches[1];
        } else {
            // Failed to match the standard '[object ClassName]'
            return toString.call(val);
        }
        if (className == 'Object') {
            // we're a user defined class or Object
            // JSON.stringify avoids problems with cycles, and is generally much
            // easier than looping through ownProperties of `val`.
            try {
                return 'Object(' + JSON.stringify(val) + ')';
            } catch (_) {
                return 'Object';
            }
        }
        // errors
        if (val instanceof Error) {
            return `${val.name}: ${val.message}\n${val.stack}`;
        }
        // TODO we could test for more things here, like `Set`s and `Map`s.
        return className;
    }
    /**
    */
    class CenterForce {

        static __wrap(ptr) {
            const obj = Object.create(CenterForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_centerforce_free(ptr);
        }
        /**
        * @returns {CenterForce}
        */
        constructor() {
            const ret = wasm.centerforce_new();
            return CenterForce.__wrap(ret);
        }
        /**
        * @returns {Force}
        */
        force() {
            const ret = wasm.centerforce_force(this.ptr);
            return Force.__wrap(ret);
        }
    }
    /**
    */
    class CollideForce {

        static __wrap(ptr) {
            const obj = Object.create(CollideForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_collideforce_free(ptr);
        }
        /**
        * @returns {CollideForce}
        */
        constructor() {
            const ret = wasm.collideforce_new();
            return CollideForce.__wrap(ret);
        }
        /**
        * @returns {Force}
        */
        force() {
            const ret = wasm.collideforce_force(this.ptr);
            return Force.__wrap(ret);
        }
        /**
        * @param {any} f
        */
        set radius(f) {
            try {
                wasm.collideforce_radius(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @returns {number}
        */
        get strength() {
            const ret = wasm.collideforce_strength(this.ptr);
            return ret;
        }
        /**
        * @param {number} value
        */
        set strength(value) {
            wasm.collideforce_set_strength(this.ptr, value);
        }
    }
    /**
    */
    class Force {

        static __wrap(ptr) {
            const obj = Object.create(Force.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_force_free(ptr);
        }
    }
    /**
    */
    class ForceDirectedEdgeBundling {

        static __wrap(ptr) {
            const obj = Object.create(ForceDirectedEdgeBundling.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_forcedirectededgebundling_free(ptr);
        }
        /**
        * @returns {ForceDirectedEdgeBundling}
        */
        constructor() {
            const ret = wasm.forcedirectededgebundling_new();
            return ForceDirectedEdgeBundling.__wrap(ret);
        }
        /**
        * @param {any} graph
        * @param {any} point_array
        * @returns {any}
        */
        call(graph, point_array) {
            const ret = wasm.forcedirectededgebundling_call(this.ptr, addHeapObject(graph), addHeapObject(point_array));
            return takeObject(ret);
        }
        /**
        * @param {any} value
        * @returns {any}
        */
        cycles(value) {
            const ret = wasm.forcedirectededgebundling_cycles(this.ptr, addHeapObject(value));
            return takeObject(ret);
        }
        /**
        * @param {any} value
        * @returns {any}
        */
        s0(value) {
            const ret = wasm.forcedirectededgebundling_s0(this.ptr, addHeapObject(value));
            return takeObject(ret);
        }
        /**
        * @param {any} value
        * @returns {any}
        */
        sStep(value) {
            const ret = wasm.forcedirectededgebundling_sStep(this.ptr, addHeapObject(value));
            return takeObject(ret);
        }
        /**
        * @param {any} value
        * @returns {any}
        */
        i0(value) {
            const ret = wasm.forcedirectededgebundling_i0(this.ptr, addHeapObject(value));
            return takeObject(ret);
        }
        /**
        * @param {any} value
        * @returns {any}
        */
        iStep(value) {
            const ret = wasm.forcedirectededgebundling_iStep(this.ptr, addHeapObject(value));
            return takeObject(ret);
        }
        /**
        * @param {any} value
        * @returns {any}
        */
        minimumEdgeCompatibility(value) {
            const ret = wasm.forcedirectededgebundling_minimumEdgeCompatibility(this.ptr, addHeapObject(value));
            return takeObject(ret);
        }
    }
    /**
    */
    class ForceDirectedGrouping {

        static __wrap(ptr) {
            const obj = Object.create(ForceDirectedGrouping.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_forcedirectedgrouping_free(ptr);
        }
        /**
        * @returns {ForceDirectedGrouping}
        */
        constructor() {
            const ret = wasm.forcedirectedgrouping_new();
            return ForceDirectedGrouping.__wrap(ret);
        }
        /**
        * @param {any} graph
        * @param {SimulationBuilder} builder
        * @param {any} new_graph
        * @returns {any}
        */
        call(graph, builder, new_graph) {
            _assertClass(builder, SimulationBuilder);
            const ptr0 = builder.ptr;
            builder.ptr = 0;
            const ret = wasm.forcedirectedgrouping_call(this.ptr, addHeapObject(graph), ptr0, addHeapObject(new_graph));
            return takeObject(ret);
        }
        /**
        * @param {any} f
        */
        set group(f) {
            try {
                wasm.forcedirectedgrouping_set_group(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set nodeWeight(f) {
            try {
                wasm.forcedirectedgrouping_set_node_weight(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set linkWeight(f) {
            try {
                wasm.forcedirectedgrouping_set_link_weight(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class GroupCenterForce {

        static __wrap(ptr) {
            const obj = Object.create(GroupCenterForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_groupcenterforce_free(ptr);
        }
        /**
        * @returns {GroupCenterForce}
        */
        constructor() {
            const ret = wasm.groupcenterforce_new();
            return GroupCenterForce.__wrap(ret);
        }
        /**
        * @returns {Force}
        */
        force() {
            const ret = wasm.groupcenterforce_force(this.ptr);
            return Force.__wrap(ret);
        }
        /**
        * @param {any} f
        */
        set group(f) {
            try {
                wasm.groupcenterforce_set_group(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set groupX(f) {
            try {
                wasm.groupcenterforce_set_group_x(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set groupY(f) {
            try {
                wasm.groupcenterforce_set_group_y(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class GroupLinkForce {

        static __wrap(ptr) {
            const obj = Object.create(GroupLinkForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_grouplinkforce_free(ptr);
        }
        /**
        * @returns {GroupLinkForce}
        */
        constructor() {
            const ret = wasm.grouplinkforce_new();
            return GroupLinkForce.__wrap(ret);
        }
        /**
        * @returns {Force}
        */
        force() {
            const ret = wasm.grouplinkforce_force(this.ptr);
            return Force.__wrap(ret);
        }
        /**
        * @param {any} f
        */
        set group(f) {
            try {
                wasm.grouplinkforce_set_group(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @returns {number}
        */
        get intraGroup() {
            const ret = wasm.grouplinkforce_intra_group(this.ptr);
            return ret;
        }
        /**
        * @param {number} value
        */
        set intraGroup(value) {
            wasm.grouplinkforce_set_intra_group(this.ptr, value);
        }
        /**
        * @returns {number}
        */
        get interGroup() {
            const ret = wasm.grouplinkforce_inter_group(this.ptr);
            return ret;
        }
        /**
        * @param {number} value
        */
        set interGroup(value) {
            wasm.grouplinkforce_set_inter_group(this.ptr, value);
        }
        /**
        * @param {any} f
        */
        set distance(f) {
            try {
                wasm.grouplinkforce_set_distance(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class GroupManyBodyForce {

        static __wrap(ptr) {
            const obj = Object.create(GroupManyBodyForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_groupmanybodyforce_free(ptr);
        }
        /**
        * @returns {GroupManyBodyForce}
        */
        constructor() {
            const ret = wasm.groupmanybodyforce_new();
            return GroupManyBodyForce.__wrap(ret);
        }
        /**
        * @returns {Force}
        */
        force() {
            const ret = wasm.groupmanybodyforce_force(this.ptr);
            return Force.__wrap(ret);
        }
        /**
        * @param {any} f
        */
        set group(f) {
            try {
                wasm.groupmanybodyforce_set_group(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set strength(f) {
            try {
                wasm.groupmanybodyforce_set_strength(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class GroupPositionForce {

        static __wrap(ptr) {
            const obj = Object.create(GroupPositionForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_grouppositionforce_free(ptr);
        }
        /**
        * @returns {GroupPositionForce}
        */
        constructor() {
            const ret = wasm.grouppositionforce_new();
            return GroupPositionForce.__wrap(ret);
        }
        /**
        * @returns {Force}
        */
        force() {
            const ret = wasm.grouppositionforce_force(this.ptr);
            return Force.__wrap(ret);
        }
        /**
        * @param {any} f
        */
        set group(f) {
            try {
                wasm.grouppositionforce_set_group(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set strength(f) {
            try {
                wasm.grouppositionforce_set_strength(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set groupX(f) {
            try {
                wasm.grouppositionforce_set_group_x(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set groupY(f) {
            try {
                wasm.grouppositionforce_set_group_y(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class LinkForce {

        static __wrap(ptr) {
            const obj = Object.create(LinkForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_linkforce_free(ptr);
        }
        /**
        * @returns {LinkForce}
        */
        constructor() {
            const ret = wasm.linkforce_new();
            return LinkForce.__wrap(ret);
        }
        /**
        * @returns {Force}
        */
        force() {
            const ret = wasm.linkforce_force(this.ptr);
            return Force.__wrap(ret);
        }
        /**
        * @param {any} f
        */
        set strength(f) {
            try {
                wasm.linkforce_set_strength(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set distance(f) {
            try {
                wasm.linkforce_distance(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class ManyBodyForce {

        static __wrap(ptr) {
            const obj = Object.create(ManyBodyForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_manybodyforce_free(ptr);
        }
        /**
        * @returns {ManyBodyForce}
        */
        constructor() {
            const ret = wasm.manybodyforce_new();
            return ManyBodyForce.__wrap(ret);
        }
        /**
        * @returns {Force}
        */
        force() {
            const ret = wasm.manybodyforce_force(this.ptr);
            return Force.__wrap(ret);
        }
        /**
        * @param {any} f
        */
        set strength(f) {
            try {
                wasm.manybodyforce_set_strength(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class PositionForce {

        static __wrap(ptr) {
            const obj = Object.create(PositionForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_positionforce_free(ptr);
        }
        /**
        * @returns {PositionForce}
        */
        constructor() {
            const ret = wasm.positionforce_new();
            return PositionForce.__wrap(ret);
        }
        /**
        * @returns {Force}
        */
        force() {
            const ret = wasm.positionforce_force(this.ptr);
            return Force.__wrap(ret);
        }
        /**
        * @param {any} f
        */
        set strength(f) {
            try {
                wasm.positionforce_set_strength(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set x(f) {
            try {
                wasm.positionforce_set_x(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set y(f) {
            try {
                wasm.positionforce_y(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class RadialForce {

        static __wrap(ptr) {
            const obj = Object.create(RadialForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_radialforce_free(ptr);
        }
        /**
        * @returns {RadialForce}
        */
        constructor() {
            const ret = wasm.radialforce_new();
            return RadialForce.__wrap(ret);
        }
        /**
        * @returns {Force}
        */
        force() {
            const ret = wasm.radialforce_force(this.ptr);
            return Force.__wrap(ret);
        }
        /**
        * @param {any} f
        */
        set strength(f) {
            try {
                wasm.radialforce_set_strength(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set radius(f) {
            try {
                wasm.radialforce_set_radius(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {number} x
        */
        set x(x) {
            wasm.radialforce_set_x(this.ptr, x);
        }
        /**
        * @param {number} y
        */
        set y(y) {
            wasm.radialforce_set_y(this.ptr, y);
        }
    }
    /**
    */
    class Simulation {

        static __wrap(ptr) {
            const obj = Object.create(Simulation.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_simulation_free(ptr);
        }
        /**
        */
        run() {
            wasm.simulation_run(this.ptr);
        }
        /**
        */
        step() {
            wasm.simulation_step(this.ptr);
        }
        /**
        * @param {number} n
        */
        stepN(n) {
            wasm.simulation_stepN(this.ptr, n);
        }
        /**
        * @returns {boolean}
        */
        isFinished() {
            const ret = wasm.simulation_isFinished(this.ptr);
            return ret !== 0;
        }
        /**
        * @param {number} alpha_start
        */
        reset(alpha_start) {
            wasm.simulation_reset(this.ptr, alpha_start);
        }
        /**
        * @param {number} u
        * @returns {number}
        */
        x(u) {
            const ret = wasm.simulation_x(this.ptr, u);
            return ret;
        }
        /**
        * @param {number} u
        * @returns {number}
        */
        y(u) {
            const ret = wasm.simulation_y(this.ptr, u);
            return ret;
        }
    }
    /**
    */
    class SimulationBuilder {

        static __wrap(ptr) {
            const obj = Object.create(SimulationBuilder.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_simulationbuilder_free(ptr);
        }
        /**
        * @returns {SimulationBuilder}
        */
        constructor() {
            const ret = wasm.simulationbuilder_new();
            return SimulationBuilder.__wrap(ret);
        }
        /**
        * @returns {SimulationBuilder}
        */
        static defaultConnected() {
            const ret = wasm.simulationbuilder_defaultConnected();
            return SimulationBuilder.__wrap(ret);
        }
        /**
        * @returns {SimulationBuilder}
        */
        static defaultNonConnected() {
            const ret = wasm.simulationbuilder_defaultNonConnected();
            return SimulationBuilder.__wrap(ret);
        }
        /**
        * @param {any} force
        * @returns {number}
        */
        add(force) {
            const ret = wasm.simulationbuilder_add(this.ptr, addHeapObject(force));
            return ret >>> 0;
        }
        /**
        * @param {number} index
        * @returns {any}
        */
        get(index) {
            const ret = wasm.simulationbuilder_get(this.ptr, index);
            return takeObject(ret);
        }
        /**
        * @param {number} index
        * @returns {any}
        */
        remove(index) {
            const ret = wasm.simulationbuilder_remove(this.ptr, index);
            return takeObject(ret);
        }
        /**
        * @param {any} graph
        * @returns {Simulation}
        */
        build(graph) {
            const ret = wasm.simulationbuilder_build(this.ptr, addHeapObject(graph));
            return Simulation.__wrap(ret);
        }
        /**
        * @param {any} graph
        * @returns {Simulation}
        */
        start(graph) {
            const ret = wasm.simulationbuilder_start(this.ptr, addHeapObject(graph));
            return Simulation.__wrap(ret);
        }
        /**
        * @returns {number}
        */
        get alphaStart() {
            const ret = wasm.simulationbuilder_alpha_start(this.ptr);
            return ret;
        }
        /**
        * @param {number} value
        */
        set alphaStart(value) {
            wasm.simulationbuilder_set_alpha_start(this.ptr, value);
        }
        /**
        * @returns {number}
        */
        get alphaMin() {
            const ret = wasm.simulationbuilder_alpha_min(this.ptr);
            return ret;
        }
        /**
        * @param {number} value
        */
        set alphaMin(value) {
            wasm.simulationbuilder_set_alpha_min(this.ptr, value);
        }
        /**
        * @returns {number}
        */
        get alphaTarget() {
            const ret = wasm.simulationbuilder_alpha_target(this.ptr);
            return ret;
        }
        /**
        * @param {number} value
        */
        set alphaTarget(value) {
            wasm.simulationbuilder_set_alpha_target(this.ptr, value);
        }
        /**
        * @returns {number}
        */
        get velocityDecay() {
            const ret = wasm.simulationbuilder_velocity_decay(this.ptr);
            return ret;
        }
        /**
        * @param {number} value
        */
        set velocityDecay(value) {
            wasm.simulationbuilder_set_velocity_decay(this.ptr, value);
        }
        /**
        * @returns {number}
        */
        get iterations() {
            const ret = wasm.simulationbuilder_iterations(this.ptr);
            return ret >>> 0;
        }
        /**
        * @param {number} value
        */
        set iterations(value) {
            wasm.simulationbuilder_set_iterations(this.ptr, value);
        }
    }
    /**
    */
    class TreemapGrouping {

        static __wrap(ptr) {
            const obj = Object.create(TreemapGrouping.prototype);
            obj.ptr = ptr;

            return obj;
        }

        free() {
            const ptr = this.ptr;
            this.ptr = 0;

            wasm.__wbg_treemapgrouping_free(ptr);
        }
        /**
        * @returns {TreemapGrouping}
        */
        constructor() {
            const ret = wasm.treemapgrouping_new();
            return TreemapGrouping.__wrap(ret);
        }
        /**
        * @param {any} graph
        * @param {number} width
        * @param {number} height
        * @returns {any}
        */
        call(graph, width, height) {
            const ret = wasm.treemapgrouping_call(this.ptr, addHeapObject(graph), width, height);
            return takeObject(ret);
        }
        /**
        * @param {any} f
        */
        set group(f) {
            try {
                wasm.treemapgrouping_set_group(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {any} f
        */
        set weight(f) {
            try {
                wasm.treemapgrouping_set_weight(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
    }

    function init(module) {
        if (typeof module === 'undefined') {
            module = (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __filename).href : (document.currentScript && document.currentScript.src || new URL('egraph.js', document.baseURI).href)).replace(/\.js$/, '_bg.wasm');
        }
        let result;
        const imports = {};
        imports.wbg = {};
        imports.wbg.__wbindgen_object_drop_ref = function(arg0) {
            takeObject(arg0);
        };
        imports.wbg.__wbindgen_string_new = function(arg0, arg1) {
            const ret = getStringFromWasm(arg0, arg1);
            return addHeapObject(ret);
        };
        imports.wbg.__wbindgen_number_new = function(arg0) {
            const ret = arg0;
            return addHeapObject(ret);
        };
        imports.wbg.__wbindgen_is_null = function(arg0) {
            const ret = getObject(arg0) === null;
            return ret;
        };
        imports.wbg.__wbindgen_is_undefined = function(arg0) {
            const ret = getObject(arg0) === undefined;
            return ret;
        };
        imports.wbg.__wbindgen_object_clone_ref = function(arg0) {
            const ret = getObject(arg0);
            return addHeapObject(ret);
        };
        imports.wbg.__wbindgen_json_parse = function(arg0, arg1) {
            const ret = JSON.parse(getStringFromWasm(arg0, arg1));
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_force_cde3c6acde9140ae = function(arg0) {
            const ret = getObject(arg0).force();
            _assertClass(ret, Force);
            const ptr0 = ret.ptr;
            ret.ptr = 0;
            return ptr0;
        };
        imports.wbg.__wbg_nodes_7b2ab9947ba2a818 = function(arg0) {
            const ret = getObject(arg0).nodes();
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_edges_4d96058d29ab6aa2 = function(arg0) {
            const ret = getObject(arg0).edges();
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_outNodes_f72535569a19bd19 = function(arg0, arg1) {
            const ret = getObject(arg0).outNodes(arg1 >>> 0);
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_inNodes_3cf96c54976106b8 = function(arg0, arg1) {
            const ret = getObject(arg0).inNodes(arg1 >>> 0);
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_nodeCount_a2c9c12b9e926808 = function(arg0) {
            const ret = getObject(arg0).nodeCount();
            return ret;
        };
        imports.wbg.__wbg_outDegree_28889d96f812bf74 = function(arg0, arg1) {
            const ret = getObject(arg0).outDegree(arg1 >>> 0);
            return ret;
        };
        imports.wbg.__wbg_inDegree_aff795860e95326d = function(arg0, arg1) {
            const ret = getObject(arg0).inDegree(arg1 >>> 0);
            return ret;
        };
        imports.wbg.__wbg_addNode_6f02f2794d3ab67b = function(arg0, arg1, arg2) {
            getObject(arg0).addNode(arg1 >>> 0, takeObject(arg2));
        };
        imports.wbg.__wbg_addEdge_0c7967766e6f429b = function(arg0, arg1, arg2, arg3) {
            getObject(arg0).addEdge(arg1 >>> 0, arg2 >>> 0, takeObject(arg3));
        };
        imports.wbg.__wbindgen_is_function = function(arg0) {
            const ret = typeof(getObject(arg0)) === 'function';
            return ret;
        };
        imports.wbg.__wbindgen_is_object = function(arg0) {
            const val = getObject(arg0);
            const ret = typeof(val) === 'object' && val !== null;
            return ret;
        };
        imports.wbg.__wbg_next_d6edd775992f0f4c = function(arg0) {
            const ret = getObject(arg0).next;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_next_0f5d9584ef1627da = function(arg0) {
            try {
                const ret = getObject(arg0).next();
                return addHeapObject(ret);
            } catch (e) {
                handleError(e);
            }
        };
        imports.wbg.__wbg_done_a9aaa2595d27799d = function(arg0) {
            const ret = getObject(arg0).done;
            return ret;
        };
        imports.wbg.__wbg_value_5aec814aeef9e8ec = function(arg0) {
            const ret = getObject(arg0).value;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_iterator_cd16caebd6db9d85 = function() {
            const ret = Symbol.iterator;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_get_f6922348004f9279 = function(arg0, arg1) {
            try {
                const ret = Reflect.get(getObject(arg0), getObject(arg1));
                return addHeapObject(ret);
            } catch (e) {
                handleError(e);
            }
        };
        imports.wbg.__wbg_call_836fa928f74337e5 = function(arg0, arg1) {
            try {
                const ret = getObject(arg0).call(getObject(arg1));
                return addHeapObject(ret);
            } catch (e) {
                handleError(e);
            }
        };
        imports.wbg.__wbg_new_b3a6d73da508ceb3 = function() {
            const ret = new Array();
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_forEach_0288520733c2c530 = function(arg0, arg1, arg2) {
            const state0 = {a: arg1, b: arg2};
            const cb0 = (arg0, arg1, arg2) => {
                const a = state0.a;
                state0.a = 0;
                try {
                    return __wbg_elem_binding0(a, state0.b, arg0, arg1, arg2);
                } finally {
                    state0.a = a;
                }
            };
            try {
                getObject(arg0).forEach(cb0);
            } finally {
                state0.a = state0.b = 0;
            }
        };
        imports.wbg.__wbg_push_7936289c733f57be = function(arg0, arg1) {
            const ret = getObject(arg0).push(getObject(arg1));
            return ret;
        };
        imports.wbg.__wbg_call_3d24f4e03e010931 = function(arg0, arg1, arg2) {
            try {
                const ret = getObject(arg0).call(getObject(arg1), getObject(arg2));
                return addHeapObject(ret);
            } catch (e) {
                handleError(e);
            }
        };
        imports.wbg.__wbg_call_d8b88ab221c982cc = function(arg0, arg1, arg2, arg3) {
            try {
                const ret = getObject(arg0).call(getObject(arg1), getObject(arg2), getObject(arg3));
                return addHeapObject(ret);
            } catch (e) {
                handleError(e);
            }
        };
        imports.wbg.__wbg_call_a0edf5238869d25a = function(arg0, arg1, arg2, arg3, arg4) {
            try {
                const ret = getObject(arg0).call(getObject(arg1), getObject(arg2), getObject(arg3), getObject(arg4));
                return addHeapObject(ret);
            } catch (e) {
                handleError(e);
            }
        };
        imports.wbg.__wbg_new_b276d8d930d44595 = function() {
            const ret = new Object();
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_get_cefed6d76a9a6478 = function(arg0, arg1) {
            try {
                const ret = Reflect.get(getObject(arg0), arg1 >>> 0);
                return addHeapObject(ret);
            } catch (e) {
                handleError(e);
            }
        };
        imports.wbg.__wbg_set_001d7d49c8e6f145 = function(arg0, arg1, arg2) {
            try {
                const ret = Reflect.set(getObject(arg0), getObject(arg1), getObject(arg2));
                return ret;
            } catch (e) {
                handleError(e);
            }
        };
        imports.wbg.__wbindgen_number_get = function(arg0, arg1) {
            const obj = getObject(arg0);
            if (typeof(obj) === 'number') return obj;
            getUint8Memory()[arg1] = 1;
            const ret = 0;
            return ret;
        };
        imports.wbg.__wbindgen_debug_string = function(arg0, arg1) {
            const ret = debugString(getObject(arg1));
            const ret0 = passStringToWasm(ret);
            const ret1 = WASM_VECTOR_LEN;
            getInt32Memory()[arg0 / 4 + 0] = ret0;
            getInt32Memory()[arg0 / 4 + 1] = ret1;
        };
        imports.wbg.__wbindgen_throw = function(arg0, arg1) {
            throw new Error(getStringFromWasm(arg0, arg1));
        };
        imports.wbg.__wbindgen_rethrow = function(arg0) {
            throw takeObject(arg0);
        };

        if (module instanceof URL || typeof module === 'string' || module instanceof Request) {

            const response = fetch(module);
            if (typeof WebAssembly.instantiateStreaming === 'function') {
                result = WebAssembly.instantiateStreaming(response, imports)
                .catch(e => {
                    console.warn("`WebAssembly.instantiateStreaming` failed. Assuming this is because your server does not serve wasm with `application/wasm` MIME type. Falling back to `WebAssembly.instantiate` which is slower. Original error:\n", e);
                    return response
                    .then(r => r.arrayBuffer())
                    .then(bytes => WebAssembly.instantiate(bytes, imports));
                });
            } else {
                result = response
                .then(r => r.arrayBuffer())
                .then(bytes => WebAssembly.instantiate(bytes, imports));
            }
        } else {

            result = WebAssembly.instantiate(module, imports)
            .then(result => {
                if (result instanceof WebAssembly.Instance) {
                    return { instance: result, module };
                } else {
                    return result;
                }
            });
        }
        return result.then(({instance, module}) => {
            wasm = instance.exports;
            init.__wbindgen_wasm_module = module;

            return wasm;
        });
    }

    const privates = new WeakMap();

    const p = (self) => privates.get(self);

    const checkNode = (graph, u) => {
      if (!p(graph).nodes.has(u)) {
        throw new Error(`Invalid node: ${u}`)
      }
    };

    class Graph {
      constructor() {
        privates.set(this, {
          nodes: new Map(),
          numNodes: 0,
          numEdges: 0
        });
      }

      node(u) {
        const nodes = p(this).nodes;
        if (nodes.get(u)) {
          return nodes.get(u).data
        }
        return null
      }

      edge(u, v) {
        const nodes = p(this).nodes;
        if (nodes.get(u) && nodes.get(u).outNodes.get(v)) {
          return nodes.get(u).outNodes.get(v)
        }
        return null
      }

      *nodes() {
        for (const key of p(this).nodes.keys()) {
          yield key;
        }
      }

      *edges() {
        for (const u of this.nodes()) {
          for (const v of this.outNodes(u)) {
            yield [u, v];
          }
        }
      }

      *outNodes(u) {
        checkNode(this, u);
        for (const key of p(this)
          .nodes.get(u)
          .outNodes.keys()) {
          yield key;
        }
      }

      *inNodes(u) {
        checkNode(this, u);
        for (const key of p(this)
          .nodes.get(u)
          .inNodes.keys()) {
          yield key;
        }
      }

      *outEdges(u) {
        for (const v of this.outNodes(u)) {
          yield [u, v];
        }
      }

      *inEdges(u) {
        for (const v of this.inNodes(u)) {
          yield [v, u];
        }
      }

      nodeCount() {
        return p(this).numNodes
      }

      edgeCount() {
        return p(this).numEdges
      }

      degree(u) {
        return this.outDegree(u) + this.inDegree(u)
      }

      outDegree(u) {
        checkNode(this, u);
        return p(this).nodes.get(u).outNodes.size
      }

      inDegree(u) {
        checkNode(this, u);
        return p(this).nodes.get(u).inNodes.size
      }

      addNode(u, obj = {}) {
        if (this.node(u)) {
          throw new Error(`Duplicated node: ${u}`)
        }
        p(this).nodes.set(u, {
          outNodes: new Map(),
          inNodes: new Map(),
          data: obj
        });
        p(this).numNodes++;
        return this
      }

      addEdge(u, v, obj = {}) {
        checkNode(this, u);
        checkNode(this, v);
        if (this.edge(u, v)) {
          throw new Error(`Duplicated edge: (${u}, ${v})`)
        }
        p(this).numEdges++;
        p(this)
          .nodes.get(u)
          .outNodes.set(v, obj);
        p(this)
          .nodes.get(v)
          .inNodes.set(u, obj);
        return this
      }

      removeNode(u) {
        for (const v of this.outNodes(u)) {
          this.removeEdge(u, v);
        }
        for (const v of this.inNodes(u)) {
          this.removeEdge(v, u);
        }
        p(this).nodes.delete(u);
        p(this).numNodes--;
        return this
      }

      removeEdge(u, v) {
        if (this.edge(u, v) === null) {
          throw Error(`Invalid edge: (${u}, ${v})`)
        }
        p(this)
          .nodes.get(u)
          .outNodes.delete(v);
        p(this)
          .nodes.get(v)
          .inNodes.delete(u);
        p(this).numEdges--;
        return this
      }

      toJSON() {
        return {
          nodes: Array.from(this.nodes()).map((u) =>
            Object.assign(this.node(u), { id: u })
          ),
          links: Array.from(this.edges()).map(([u, v]) =>
            Object.assign(this.edge(u, v), { source: u, target: v })
          )
        }
      }

      toString() {
        return JSON.stringify(this.toJSON())
      }
    }

    exports.CenterForce = CenterForce;
    exports.CollideForce = CollideForce;
    exports.Force = Force;
    exports.ForceDirectedEdgeBundling = ForceDirectedEdgeBundling;
    exports.ForceDirectedGrouping = ForceDirectedGrouping;
    exports.Graph = Graph;
    exports.GroupCenterForce = GroupCenterForce;
    exports.GroupLinkForce = GroupLinkForce;
    exports.GroupManyBodyForce = GroupManyBodyForce;
    exports.GroupPositionForce = GroupPositionForce;
    exports.LinkForce = LinkForce;
    exports.ManyBodyForce = ManyBodyForce;
    exports.PositionForce = PositionForce;
    exports.RadialForce = RadialForce;
    exports.Simulation = Simulation;
    exports.SimulationBuilder = SimulationBuilder;
    exports.TreemapGrouping = TreemapGrouping;
    exports.aggregateEdges = aggregateEdges;
    exports.aggregateNodes = aggregateNodes;
    exports.connected_components = connected_components;
    exports.init = init;
    exports.longestPathRanking = longestPathRanking;
    exports.minWidthRanking = minWidthRanking;
    exports.muQuasiBicliques = muQuasiBicliques;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
